// ADC driver for nRF52840 | gat 11/04/23
#include "nrf52840_bitfields.h"
#include "nrf52840.h"
#include "ADC.h"
#include <stddef.h>

#define ADC_CHANNEL_NB 8

void ADC_Init()
{
	NRF_SAADC->INTENSET = 1U<<SAADC_INTEN_END_Pos;
	NRF_SAADC->RESOLUTION = SAADC_RESOLUTION_VAL_12bit; //12 bits
	NRF_SAADC->SAMPLERATE = SAADC_SAMPLERATE_MODE_Task; //triggered by task(for scan mode)
	//NRF_SAADC->OVERSAMPLE = 2U; // OV 4x
	NRF_SAADC->ENABLE = 1U;
}

void ADC_Start(void)
{
	NRF_SAADC->TASKS_START = 1U;
}

void ADC_Stop(void)
{
	NRF_SAADC->TASKS_STOP = 1U;
}

void ADC_Sample(void* p_data)
{
	NRF_SAADC->EVENTS_END = 0;
	NRF_SAADC->EVENTS_DONE = 0;
	NRF_SAADC->EVENTS_RESULTDONE = 0;
	NRF_SAADC->RESULT.PTR = (unsigned int)p_data;
	NRF_SAADC->TASKS_START = 1;
	NRF_SAADC->TASKS_SAMPLE = 1U;
}

static void (*EndOfConversion_Callback)(void) = NULL;

void ADC_Register_Callback(void (*user_Callback)(void))
{
	EndOfConversion_Callback = user_Callback;
}


ADC_Status ADC_Add_Channel(ADC_Analog_Input AnalogInput, ADC_Gain Gain)
{
	unsigned int channel = -1;
	NRF_SAADC->ENABLE = 0;
	for(unsigned int i = 0; i < ADC_CHANNEL_NB; i++) // Assign a channel
		if(NRF_SAADC->CH[i].PSELP == SAADC_CH_PSELN_PSELN_NC) { channel = i; break; }
	if(channel == -1) return ADC_Status_No_Chann_Available;
	NRF_SAADC->CH[channel].PSELP = (unsigned int)AnalogInput;
	NRF_SAADC->CH[channel].CONFIG = (SAADC_CH_CONFIG_RESP_Pulldown << SAADC_CH_CONFIG_RESP_Pos)
											+ (SAADC_CH_CONFIG_REFSEL_Internal << SAADC_CH_CONFIG_REFSEL_Pos) // 0.6V as reference
											+ (SAADC_CH_CONFIG_TACQ_15us << SAADC_CH_CONFIG_TACQ_Pos) // 15 us sample time
											+ (SAADC_CH_CONFIG_GAIN_Gain1_4 << SAADC_CH_CONFIG_GAIN_Pos); // gain 1/6 (input range = 3.6V)
	NRF_SAADC->RESULT.MAXCNT++;
	NRF_SAADC->ENABLE = 1;
	return ADC_Status_OK;
}

ADC_Status ADC_Remove_Channel(ADC_Analog_Input AnalogInput)
{
	for(unsigned int i = 0; i < ADC_CHANNEL_NB; i++)
		if(NRF_SAADC->CH[i].PSELP == (unsigned int)AnalogInput)
		{
			NRF_SAADC->CH[i].PSELP = SAADC_CH_PSELN_PSELN_NC;
			NRF_SAADC->RESULT.MAXCNT--;
			return ADC_Status_OK;
		}
	return ADC_Status_Chann_Not_Found;
}

void SAADC_IRQHandler(void)
{
	NRF_SAADC->EVENTS_END = 0;
	if(EndOfConversion_Callback != NULL) EndOfConversion_Callback(); 
}
