// RTC driver for nRF52840 | gat 11/04/23
#include "nrf52840.h"
#include "RTC.h"

static NRF_RTC_Type* Get_RTC(RTC_Channel ch)
{
	switch(ch)
	{
		default:
		case RTC0 :
			return NRF_RTC0;
		case RTC1 :
			return NRF_RTC1;
		case RTC2 :
			return NRF_RTC2;
	}
}

static void (*Frame_Received_Callback[3])(void);

void RTC_Setup(RTC_Config cfg, void (*p_Frame_Received_Function)(void))
{
	NRF_RTC_Type* rtc = Get_RTC(cfg.channel);
	rtc->TASKS_STOP = 1;
	rtc->INTENSET = (unsigned int)cfg.irq;	
	rtc->PRESCALER = (unsigned int)cfg.prescaler;

	Frame_Received_Callback[(unsigned char)cfg.channel] = p_Frame_Received_Function;
}

void RTC_Clear(RTC_Channel chann)
{
	NRF_RTC_Type* rtc = Get_RTC(chann);
	rtc->EVENTS_COMPARE[0] = 0;
}

void RTC_Start(RTC_Channel chann, unsigned int time_period)
{
	NRF_RTC_Type* rtc = Get_RTC(chann);
	rtc->TASKS_CLEAR = 1;
	rtc->EVENTS_COMPARE[0] = 0;
	rtc->CC[0] = time_period;
	rtc->TASKS_START = 1;
}

void RTC0_IRQHandler(void)
{
	NRF_RTC0->EVENTS_COMPARE[0] = 0;
	if(Frame_Received_Callback[0] != 0) Frame_Received_Callback[0]();
}

void RTC2_IRQHandler(void)
{
	NRF_RTC2->EVENTS_COMPARE[0] = 0;
	if(Frame_Received_Callback[2] != 0) Frame_Received_Callback[2]();
}

