// SPI Master driver for nRF52840 | gat 11/04/23
#include "SPIM.h"
#include "nrf52840.h"

static NRF_SPIM_Type* GetReg(SPIM_Channel ch)
{
	switch(ch)
	{
		default:
		case SPIM0:
		return NRF_SPIM0;
		case SPIM1:
		return NRF_SPIM1;
		case SPIM2:
		return NRF_SPIM2; 
		case SPIM3:
		return NRF_SPIM3;
	}
}

void SPIM_Setup(SPIM_Config cfg)
{
	NRF_SPIM_Type* reg = GetReg(cfg.Channel);
	reg->PSEL.SCK = cfg.CLK_Pin + (cfg.CLK_Port<<5U);
	reg->PSEL.MOSI = cfg.MOSI_Pin + (cfg.MOSI_Port<<5U);
	reg->PSEL.MISO = cfg.MISO_Pin + (cfg.MISO_Port<<5U);
	reg->FREQUENCY = cfg.Freq;
	reg->CONFIG = (cfg.Endian) | (cfg.CPHA << 1U) | (cfg.CPOL << 2U);
	reg->ENABLE = 7;
}

void SPIM_Tx(SPIM_Channel ch, void* data, unsigned short length)
{
	NRF_SPIM_Type* reg = GetReg(ch);
	reg->ENABLE = 7;
	reg->RXD.MAXCNT = 0;
	reg->TXD.PTR = (uint32_t)data;
	reg->TXD.MAXCNT = length;
	reg->TASKS_START = 1;
}

void SPIM0_IRQ(void)
{
	NRF_SPIM0->EVENTS_ENDRX = 0;
	NRF_SPIM0->EVENTS_ENDTX = 0;
	NRF_SPIM0->EVENTS_STARTED = 0;
	NRF_SPIM0->EVENTS_STOPPED = 0;
	NRF_SPIM0->EVENTS_END = 0;
}

void SPIM1_IRQ(void)
{
	NRF_SPIM1->EVENTS_ENDRX = 0;
	NRF_SPIM1->EVENTS_ENDTX = 0;
	NRF_SPIM1->EVENTS_STARTED = 0;
	NRF_SPIM1->EVENTS_STOPPED = 0;
	NRF_SPIM1->EVENTS_END = 0;
}
/*
void SPIM2_SPIS2_SPI2_IRQHandler(void)
{
	NRF_SPIM2->EVENTS_ENDRX = 0;
	NRF_SPIM2->EVENTS_ENDTX = 0;
	NRF_SPIM2->EVENTS_STARTED = 0;
	NRF_SPIM2->EVENTS_STOPPED = 0;
	NRF_SPIM2->EVENTS_END = 0;
}
*/