#include "neb.hpp"
#include "cough.hpp"
#include "GPIO.h"
#include "HSIS.h"
#include "PWM.h"
#include "ports.h"
#include "ADC.h"

#define DETECTION_UNLOCK_MS 1000
#define ADC_VALUE_250mV 500
#define ADC_VALUE_10V 2000

static bool Neb_running = false;
static short ADC_Buffer;
static TaskHandle_t current_task;

void Neb::Setup(void)
{
	PWM_Config pwm_cfg = {
		.Channel = NEB_PWM_Channel,
		.PWM_Pin = NEB_PWM_Pin,
		.PWM_Port = NEB_PWM_Port
	};
	PWM_Setup(pwm_cfg);
	Disable();
}

static void EndOfConversion(void)
{
	BaseType_t xHigherPriorityTaskWoken;
	vTaskNotifyGiveFromISR(current_task, &xHigherPriorityTaskWoken);
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

static void Sample_Neb_Supply(void)
{
	(void)ADC_Add_Channel(ADC_Analog_Input_2, ADC_Input_Range_Vdd);
	ADC_Init();
	current_task = xTaskGetCurrentTaskHandle();
	ADC_Register_Callback(EndOfConversion);
	ADC_Sample(&ADC_Buffer);
	ulTaskNotifyTake(pdTRUE, DETECTION_UNLOCK_MS);
	(void)ADC_Remove_Channel(ADC_Analog_Input_2);
}

bool Neb::Detect(void)
{
	Sample_Neb_Supply();
	return ADC_Buffer > ADC_VALUE_250mV;
}

void Neb::Enable(void)
{
	io_set(EN_NEB_Port, EN_NEB_Pin);
	Neb_running = true;
	PWM_Start(NEB_PWM_Channel);
}

void Neb::Disable(void)
{
	io_clr(EN_NEB_Port, EN_NEB_Pin);
	Neb_running = false;
	PWM_Stop(NEB_PWM_Channel);
}

static NebInfo Current { .frequency = 40, .duty_cycle = 50 };

bool Neb::Update(NebInfo config)
{
	if(config.duty_cycle == 0)
	{
		Disable();
		return false;
	}
	PWM_Update(NEB_PWM_Channel, config.frequency, config.duty_cycle);
	Current.frequency = config.frequency;
	Current.duty_cycle = config.duty_cycle;
	return true;
}

bool Neb::Detect_Supply(void)
{
	// TODO do not stop cough to get a neb supply sample
	Cough::Stop();
	Sample_Neb_Supply();
	Cough::Start();
	return ADC_Buffer > ADC_VALUE_10V;
}

NebInfo Neb::GetInfo(void) { return Current; }
